﻿/*-------------------------------------------------------------------------
 Cipher algorithm interface
 
 File name   : Cipher.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : Sep/03/2008
 Desgined by : Takeshi Sugawara
 Modified by : Toshihiro Katashita
 
 
 Copyright (C) 2008 AIST and Tohoku Univ.
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST and Tohoku University ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Cipher
{
	//================================================================ AES
	public class AES : CipherInterface
	{
		//************************************************ Variables
		#region Declaration of tables
		static private readonly Byte[] Sbox = new Byte[256] {
            0x63, 0x7c, 0x77, 0x7b, 0xf2, 0x6b, 0x6f, 0xc5, 0x30, 0x01, 0x67, 0x2b, 0xfe, 0xd7, 0xab, 0x76,
            0xca, 0x82, 0xc9, 0x7d, 0xfa, 0x59, 0x47, 0xf0, 0xad, 0xd4, 0xa2, 0xaf, 0x9c, 0xa4, 0x72, 0xc0,
            0xb7, 0xfd, 0x93, 0x26, 0x36, 0x3f, 0xf7, 0xcc, 0x34, 0xa5, 0xe5, 0xf1, 0x71, 0xd8, 0x31, 0x15,
            0x04, 0xc7, 0x23, 0xc3, 0x18, 0x96, 0x05, 0x9a, 0x07, 0x12, 0x80, 0xe2, 0xeb, 0x27, 0xb2, 0x75,
            0x09, 0x83, 0x2c, 0x1a, 0x1b, 0x6e, 0x5a, 0xa0, 0x52, 0x3b, 0xd6, 0xb3, 0x29, 0xe3, 0x2f, 0x84,
            0x53, 0xd1, 0x00, 0xed, 0x20, 0xfc, 0xb1, 0x5b, 0x6a, 0xcb, 0xbe, 0x39, 0x4a, 0x4c, 0x58, 0xcf,
            0xd0, 0xef, 0xaa, 0xfb, 0x43, 0x4d, 0x33, 0x85, 0x45, 0xf9, 0x02, 0x7f, 0x50, 0x3c, 0x9f, 0xa8,
            0x51, 0xa3, 0x40, 0x8f, 0x92, 0x9d, 0x38, 0xf5, 0xbc, 0xb6, 0xda, 0x21, 0x10, 0xff, 0xf3, 0xd2,
            0xcd, 0x0c, 0x13, 0xec, 0x5f, 0x97, 0x44, 0x17, 0xc4, 0xa7, 0x7e, 0x3d, 0x64, 0x5d, 0x19, 0x73,
            0x60, 0x81, 0x4f, 0xdc, 0x22, 0x2a, 0x90, 0x88, 0x46, 0xee, 0xb8, 0x14, 0xde, 0x5e, 0x0b, 0xdb,
            0xe0, 0x32, 0x3a, 0x0a, 0x49, 0x06, 0x24, 0x5c, 0xc2, 0xd3, 0xac, 0x62, 0x91, 0x95, 0xe4, 0x79,
            0xe7, 0xc8, 0x37, 0x6d, 0x8d, 0xd5, 0x4e, 0xa9, 0x6c, 0x56, 0xf4, 0xea, 0x65, 0x7a, 0xae, 0x08,
            0xba, 0x78, 0x25, 0x2e, 0x1c, 0xa6, 0xb4, 0xc6, 0xe8, 0xdd, 0x74, 0x1f, 0x4b, 0xbd, 0x8b, 0x8a,
            0x70, 0x3e, 0xb5, 0x66, 0x48, 0x03, 0xf6, 0x0e, 0x61, 0x35, 0x57, 0xb9, 0x86, 0xc1, 0x1d, 0x9e,
            0xe1, 0xf8, 0x98, 0x11, 0x69, 0xd9, 0x8e, 0x94, 0x9b, 0x1e, 0x87, 0xe9, 0xce, 0x55, 0x28, 0xdf,
            0x8c, 0xa1, 0x89, 0x0d, 0xbf, 0xe6, 0x42, 0x68, 0x41, 0x99, 0x2d, 0x0f, 0xb0, 0x54, 0xbb, 0x16 };

		static private readonly Byte[] InvSbox = new Byte[256] {
            0x52, 0x09, 0x6a, 0xd5, 0x30, 0x36, 0xa5, 0x38, 0xbf, 0x40, 0xa3, 0x9e, 0x81, 0xf3, 0xd7, 0xfb,
            0x7c, 0xe3, 0x39, 0x82, 0x9b, 0x2f, 0xff, 0x87, 0x34, 0x8e, 0x43, 0x44, 0xc4, 0xde, 0xe9, 0xcb,
            0x54, 0x7b, 0x94, 0x32, 0xa6, 0xc2, 0x23, 0x3d, 0xee, 0x4c, 0x95, 0x0b, 0x42, 0xfa, 0xc3, 0x4e,
            0x08, 0x2e, 0xa1, 0x66, 0x28, 0xd9, 0x24, 0xb2, 0x76, 0x5b, 0xa2, 0x49, 0x6d, 0x8b, 0xd1, 0x25,
            0x72, 0xf8, 0xf6, 0x64, 0x86, 0x68, 0x98, 0x16, 0xd4, 0xa4, 0x5c, 0xcc, 0x5d, 0x65, 0xb6, 0x92,
            0x6c, 0x70, 0x48, 0x50, 0xfd, 0xed, 0xb9, 0xda, 0x5e, 0x15, 0x46, 0x57, 0xa7, 0x8d, 0x9d, 0x84,
            0x90, 0xd8, 0xab, 0x00, 0x8c, 0xbc, 0xd3, 0x0a, 0xf7, 0xe4, 0x58, 0x05, 0xb8, 0xb3, 0x45, 0x06,
            0xd0, 0x2c, 0x1e, 0x8f, 0xca, 0x3f, 0x0f, 0x02, 0xc1, 0xaf, 0xbd, 0x03, 0x01, 0x13, 0x8a, 0x6b,
            0x3a, 0x91, 0x11, 0x41, 0x4f, 0x67, 0xdc, 0xea, 0x97, 0xf2, 0xcf, 0xce, 0xf0, 0xb4, 0xe6, 0x73,
            0x96, 0xac, 0x74, 0x22, 0xe7, 0xad, 0x35, 0x85, 0xe2, 0xf9, 0x37, 0xe8, 0x1c, 0x75, 0xdf, 0x6e,
            0x47, 0xf1, 0x1a, 0x71, 0x1d, 0x29, 0xc5, 0x89, 0x6f, 0xb7, 0x62, 0x0e, 0xaa, 0x18, 0xbe, 0x1b,
            0xfc, 0x56, 0x3e, 0x4b, 0xc6, 0xd2, 0x79, 0x20, 0x9a, 0xdb, 0xc0, 0xfe, 0x78, 0xcd, 0x5a, 0xf4,
            0x1f, 0xdd, 0xa8, 0x33, 0x88, 0x07, 0xc7, 0x31, 0xb1, 0x12, 0x10, 0x59, 0x27, 0x80, 0xec, 0x5f,
            0x60, 0x51, 0x7f, 0xa9, 0x19, 0xb5, 0x4a, 0x0d, 0x2d, 0xe5, 0x7a, 0x9f, 0x93, 0xc9, 0x9c, 0xef,
            0xa0, 0xe0, 0x3b, 0x4d, 0xae, 0x2a, 0xf5, 0xb0, 0xc8, 0xeb, 0xbb, 0x3c, 0x83, 0x53, 0x99, 0x61,
            0x17, 0x2b, 0x04, 0x7e, 0xba, 0x77, 0xd6, 0x26, 0xe1, 0x69, 0x14, 0x63, 0x55, 0x21, 0x0c, 0x7d };

		static private readonly UInt32[] Rcon = new UInt32[10]{
            0x01000000, 0x02000000, 0x04000000, 0x08000000,
            0x10000000, 0x20000000, 0x40000000, 0x80000000,
            0x1b000000, 0x36000000 };

		static private readonly UInt32[] Tbox = new UInt32[256] {
            0xC66363A5, 0xF87C7C84, 0xEE777799, 0xF67B7B8D, 0xFFF2F20D, 0xD66B6BBD,
            0xDE6F6FB1, 0x91C5C554, 0x60303050, 0x02010103, 0xCE6767A9, 0x562B2B7D,
            0xE7FEFE19, 0xB5D7D762, 0x4DABABE6, 0xEC76769A, 0x8FCACA45, 0x1F82829D,
            0x89C9C940, 0xFA7D7D87, 0xEFFAFA15, 0xB25959EB, 0x8E4747C9, 0xFBF0F00B,
            0x41ADADEC, 0xB3D4D467, 0x5FA2A2FD, 0x45AFAFEA, 0x239C9CBF, 0x53A4A4F7,
            0xE4727296, 0x9BC0C05B, 0x75B7B7C2, 0xE1FDFD1C, 0x3D9393AE, 0x4C26266A,
            0x6C36365A, 0x7E3F3F41, 0xF5F7F702, 0x83CCCC4F, 0x6834345C, 0x51A5A5F4,
            0xD1E5E534, 0xF9F1F108, 0xE2717193, 0xABD8D873, 0x62313153, 0x2A15153F,
            0x0804040C, 0x95C7C752, 0x46232365, 0x9DC3C35E, 0x30181828, 0x379696A1,
            0x0A05050F, 0x2F9A9AB5, 0x0E070709, 0x24121236, 0x1B80809B, 0xDFE2E23D,
            0xCDEBEB26, 0x4E272769, 0x7FB2B2CD, 0xEA75759F, 0x1209091B, 0x1D83839E,
            0x582C2C74, 0x341A1A2E, 0x361B1B2D, 0xDC6E6EB2, 0xB45A5AEE, 0x5BA0A0FB,
            0xA45252F6, 0x763B3B4D, 0xB7D6D661, 0x7DB3B3CE, 0x5229297B, 0xDDE3E33E,
            0x5E2F2F71, 0x13848497, 0xA65353F5, 0xB9D1D168, 0x00000000, 0xC1EDED2C,
            0x40202060, 0xE3FCFC1F, 0x79B1B1C8, 0xB65B5BED, 0xD46A6ABE, 0x8DCBCB46,
            0x67BEBED9, 0x7239394B, 0x944A4ADE, 0x984C4CD4, 0xB05858E8, 0x85CFCF4A,
            0xBBD0D06B, 0xC5EFEF2A, 0x4FAAAAE5, 0xEDFBFB16, 0x864343C5, 0x9A4D4DD7,
            0x66333355, 0x11858594, 0x8A4545CF, 0xE9F9F910, 0x04020206, 0xFE7F7F81,
            0xA05050F0, 0x783C3C44, 0x259F9FBA, 0x4BA8A8E3, 0xA25151F3, 0x5DA3A3FE,
            0x804040C0, 0x058F8F8A, 0x3F9292AD, 0x219D9DBC, 0x70383848, 0xF1F5F504,
            0x63BCBCDF, 0x77B6B6C1, 0xAFDADA75, 0x42212163, 0x20101030, 0xE5FFFF1A,
            0xFDF3F30E, 0xBFD2D26D, 0x81CDCD4C, 0x180C0C14, 0x26131335, 0xC3ECEC2F,
            0xBE5F5FE1, 0x359797A2, 0x884444CC, 0x2E171739, 0x93C4C457, 0x55A7A7F2,
            0xFC7E7E82, 0x7A3D3D47, 0xC86464AC, 0xBA5D5DE7, 0x3219192B, 0xE6737395,
            0xC06060A0, 0x19818198, 0x9E4F4FD1, 0xA3DCDC7F, 0x44222266, 0x542A2A7E,
            0x3B9090AB, 0x0B888883, 0x8C4646CA, 0xC7EEEE29, 0x6BB8B8D3, 0x2814143C,
            0xA7DEDE79, 0xBC5E5EE2, 0x160B0B1D, 0xADDBDB76, 0xDBE0E03B, 0x64323256,
            0x743A3A4E, 0x140A0A1E, 0x924949DB, 0x0C06060A, 0x4824246C, 0xB85C5CE4,
            0x9FC2C25D, 0xBDD3D36E, 0x43ACACEF, 0xC46262A6, 0x399191A8, 0x319595A4,
            0xD3E4E437, 0xF279798B, 0xD5E7E732, 0x8BC8C843, 0x6E373759, 0xDA6D6DB7,
            0x018D8D8C, 0xB1D5D564, 0x9C4E4ED2, 0x49A9A9E0, 0xD86C6CB4, 0xAC5656FA,
            0xF3F4F407, 0xCFEAEA25, 0xCA6565AF, 0xF47A7A8E, 0x47AEAEE9, 0x10080818,
            0x6FBABAD5, 0xF0787888, 0x4A25256F, 0x5C2E2E72, 0x381C1C24, 0x57A6A6F1,
            0x73B4B4C7, 0x97C6C651, 0xCBE8E823, 0xA1DDDD7C, 0xE874749C, 0x3E1F1F21,
            0x964B4BDD, 0x61BDBDDC, 0x0D8B8B86, 0x0F8A8A85, 0xE0707090, 0x7C3E3E42,
            0x71B5B5C4, 0xCC6666AA, 0x904848D8, 0x06030305, 0xF7F6F601, 0x1C0E0E12,
            0xC26161A3, 0x6A35355F, 0xAE5757F9, 0x69B9B9D0, 0x17868691, 0x99C1C158,
            0x3A1D1D27, 0x279E9EB9, 0xD9E1E138, 0xEBF8F813, 0x2B9898B3, 0x22111133,
            0xD26969BB, 0xA9D9D970, 0x078E8E89, 0x339494A7, 0x2D9B9BB6, 0x3C1E1E22,
            0x15878792, 0xC9E9E920, 0x87CECE49, 0xAA5555FF, 0x50282878, 0xA5DFDF7A,
            0x038C8C8F, 0x59A1A1F8, 0x09898980, 0x1A0D0D17, 0x65BFBFDA, 0xD7E6E631,
            0x844242C6, 0xD06868B8, 0x824141C3, 0x299999B0, 0x5A2D2D77, 0x1E0F0F11,
            0x7BB0B0CB, 0xA85454FC, 0x6DBBBBD6, 0x2C16163A};

		static private readonly UInt32[] InvTbox = new UInt32[256] {
            0x51F4A750, 0x7E416553, 0x1A17A4C3, 0x3A275E96, 0x3BAB6BCB, 0x1F9D45F1,
            0xACFA58AB, 0x4BE30393, 0x2030FA55, 0xAD766DF6, 0x88CC7691, 0xF5024C25,
            0x4FE5D7FC, 0xC52ACBD7, 0x26354480, 0xB562A38F, 0xDEB15A49, 0x25BA1B67,
            0x45EA0E98, 0x5DFEC0E1, 0xC32F7502, 0x814CF012, 0x8D4697A3, 0x6BD3F9C6,
            0x038F5FE7, 0x15929C95, 0xBF6D7AEB, 0x955259DA, 0xD4BE832D, 0x587421D3,
            0x49E06929, 0x8EC9C844, 0x75C2896A, 0xF48E7978, 0x99583E6B, 0x27B971DD,
            0xBEE14FB6, 0xF088AD17, 0xC920AC66, 0x7DCE3AB4, 0x63DF4A18, 0xE51A3182,
            0x97513360, 0x62537F45, 0xB16477E0, 0xBB6BAE84, 0xFE81A01C, 0xF9082B94,
            0x70486858, 0x8F45FD19, 0x94DE6C87, 0x527BF8B7, 0xAB73D323, 0x724B02E2,
            0xE31F8F57, 0x6655AB2A, 0xB2EB2807, 0x2FB5C203, 0x86C57B9A, 0xD33708A5,
            0x302887F2, 0x23BFA5B2, 0x02036ABA, 0xED16825C, 0x8ACF1C2B, 0xA779B492,
            0xF307F2F0, 0x4E69E2A1, 0x65DAF4CD, 0x0605BED5, 0xD134621F, 0xC4A6FE8A,
            0x342E539D, 0xA2F355A0, 0x058AE132, 0xA4F6EB75, 0x0B83EC39, 0x4060EFAA,
            0x5E719F06, 0xBD6E1051, 0x3E218AF9, 0x96DD063D, 0xDD3E05AE, 0x4DE6BD46,
            0x91548DB5, 0x71C45D05, 0x0406D46F, 0x605015FF, 0x1998FB24, 0xD6BDE997,
            0x894043CC, 0x67D99E77, 0xB0E842BD, 0x07898B88, 0xE7195B38, 0x79C8EEDB,
            0xA17C0A47, 0x7C420FE9, 0xF8841EC9, 0x00000000, 0x09808683, 0x322BED48,
            0x1E1170AC, 0x6C5A724E, 0xFD0EFFFB, 0x0F853856, 0x3DAED51E, 0x362D3927,
            0x0A0FD964, 0x685CA621, 0x9B5B54D1, 0x24362E3A, 0x0C0A67B1, 0x9357E70F,
            0xB4EE96D2, 0x1B9B919E, 0x80C0C54F, 0x61DC20A2, 0x5A774B69, 0x1C121A16,
            0xE293BA0A, 0xC0A02AE5, 0x3C22E043, 0x121B171D, 0x0E090D0B, 0xF28BC7AD,
            0x2DB6A8B9, 0x141EA9C8, 0x57F11985, 0xAF75074C, 0xEE99DDBB, 0xA37F60FD,
            0xF701269F, 0x5C72F5BC, 0x44663BC5, 0x5BFB7E34, 0x8B432976, 0xCB23C6DC,
            0xB6EDFC68, 0xB8E4F163, 0xD731DCCA, 0x42638510, 0x13972240, 0x84C61120,
            0x854A247D, 0xD2BB3DF8, 0xAEF93211, 0xC729A16D, 0x1D9E2F4B, 0xDCB230F3,
            0x0D8652EC, 0x77C1E3D0, 0x2BB3166C, 0xA970B999, 0x119448FA, 0x47E96422,
            0xA8FC8CC4, 0xA0F03F1A, 0x567D2CD8, 0x223390EF, 0x87494EC7, 0xD938D1C1,
            0x8CCAA2FE, 0x98D40B36, 0xA6F581CF, 0xA57ADE28, 0xDAB78E26, 0x3FADBFA4,
            0x2C3A9DE4, 0x5078920D, 0x6A5FCC9B, 0x547E4662, 0xF68D13C2, 0x90D8B8E8,
            0x2E39F75E, 0x82C3AFF5, 0x9F5D80BE, 0x69D0937C, 0x6FD52DA9, 0xCF2512B3,
            0xC8AC993B, 0x10187DA7, 0xE89C636E, 0xDB3BBB7B, 0xCD267809, 0x6E5918F4,
            0xEC9AB701, 0x834F9AA8, 0xE6956E65, 0xAAFFE67E, 0x21BCCF08, 0xEF15E8E6,
            0xBAE79BD9, 0x4A6F36CE, 0xEA9F09D4, 0x29B07CD6, 0x31A4B2AF, 0x2A3F2331,
            0xC6A59430, 0x35A266C0, 0x744EBC37, 0xFC82CAA6, 0xE090D0B0, 0x33A7D815,
            0xF104984A, 0x41ECDAF7, 0x7FCD500E, 0x1791F62F, 0x764DD68D, 0x43EFB04D,
            0xCCAA4D54, 0xE49604DF, 0x9ED1B5E3, 0x4C6A881B, 0xC12C1FB8, 0x4665517F,
            0x9D5EEA04, 0x018C355D, 0xFA877473, 0xFB0B412E, 0xB3671D5A, 0x92DBD252,
            0xE9105633, 0x6DD64713, 0x9AD7618C, 0x37A10C7A, 0x59F8148E, 0xEB133C89,
            0xCEA927EE, 0xB761C935, 0xE11CE5ED, 0x7A47B13C, 0x9CD2DF59, 0x55F2733F,
            0x1814CE79, 0x73C737BF, 0x53F7CDEA, 0x5FFDAA5B, 0xDF3D6F14, 0x7844DB86,
            0xCAAFF381, 0xB968C43E, 0x3824342C, 0xC2A3405F, 0x161DC372, 0xBCE2250C,
            0x283C498B, 0xFF0D9541, 0x39A80171, 0x080CB3DE, 0xD8B4E49C, 0x6456C190,
            0x7BCB8461, 0xD532B670, 0x486C5C74, 0xD0B85742 };
		#endregion

		#region Declaration of member variables
		private UInt32[] SecretKey;
		private UInt32[] RoundKey;
		private UInt32[] InvRoundKey;
		protected UInt32[] IntermediateData;
		#endregion

		//------------------------------------------------ Constructor
		public AES()
		{
			this.SecretKey = new UInt32[4];
			this.RoundKey = new UInt32[44];
			this.InvRoundKey = new UInt32[44];
			this.IntermediateData = new UInt32[4];
		}


		//************************************************ Interface
		//------------------------------------------------
		public int getKeyWidth() {return 16;}
		public int getDatWidth() {return 16;}

		//------------------------------------------------ setKey()
		public void setkey(byte[] key)
		{
			for (int i = 0; i < 4; i++)
			{
				byte[] k = new byte[4];
				for (int j = 0; j < 4; j++) k[j] = key[i * 4 + j];
				this.SecretKey[i] = Utility.ByteArrayToUInt32(k);
			}
			KeySchedule();
		}

		//------------------------------------------------ encrypt()
		public byte[] encrypt(byte[] pt)
		{
			for (int i = 0; i < 4; i++)
			{
				byte[] t = new byte[4];
				for (int j = 0; j < 4; j++) t[j] = pt[i * 4 + j];
				this.IntermediateData[i] = Utility.ByteArrayToUInt32(t);
			}

			DoEncrypt();

			byte[] ct = new byte[16];
			for (int i = 0; i < 4; i++) {
				byte[] t = Utility.UInt32ToByteArray(this.IntermediateData[i]);
				for (int j = 0; j < 4; j++) ct[i * 4 + j] = t[j];
			}
            return ct;
		}

		//------------------------------------------------ decrypt()
		public byte[] decrypt(byte[] ct) {
			for (int i = 0; i < 4; i++)
			{
				byte[] t = new byte[4];
				for (int j = 0; j < 4; j++) t[j] = ct[i * 4 + j];
				this.IntermediateData[i] = Utility.ByteArrayToUInt32(t);
			}

			DoDecrypt();

			byte[] pt = new byte[16];
			for (int i = 0; i < 4; i++) {
				byte[] t = Utility.UInt32ToByteArray(this.IntermediateData[i]);
				for (int j = 0; j < 4; j++) pt[i * 4 + j] = t[j];
			}
			return pt;
		}

		//************************************************ Cipher functions
				#region Functions start with "Apply*" transform this.IntermediateData[].

				public void ApplyShiftRows() {
            IntermediateData = ShiftRows(IntermediateData);
        }

        public void ApplyInvShiftRows() {
            IntermediateData = InvShiftRows(IntermediateData);
        }

        public void ApplySubByteMixColumn() {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = SubByteMixColumn(IntermediateData[i]);
            }
        }

        public void ApplyInvSubByteMixColumn() {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = InvSubByteMixColumn(IntermediateData[i]);
            }
        }

        public void ApplyAddRoundKey(int index) {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = IntermediateData[i] ^ RoundKey[i+index];
            }
        }

        public void ApplyInvAddRoundKey(int index) {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = IntermediateData[i] ^ InvRoundKey[i + index];
            }
        }

        public void ApplySubByte() {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = SubWord(IntermediateData[i]);
            }
        }

        public void ApplyInvSubByte() {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = InvSubWord(IntermediateData[i]);
            }
        }

        public void ApplyMixColumn() {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = MixColumn(IntermediateData[i]);
            }
        }

        public void ApplyInvMixColumn() {
            for (int i = 0; i < 4; ++i) {
                IntermediateData[i] = InvMixColumn(IntermediateData[i]);
            }
        }
        #endregion

				#region Encryption / Decryption interface
				public UInt32 [] Encrypt(UInt32[] PlainText) {
            if (PlainText.Length != 4) {
                throw new System.Exception("Invalid length of argument array in Encrypt");
            }
            else {
                for (int i = 0; i < 4; ++i) {
                    this.IntermediateData[i] = PlainText[i];
                }
                DoEncrypt();
            }
            UInt32[] temp = new UInt32[4];
            for (int i = 0; i < 4; ++i) temp[i] = this.IntermediateData[i];
            return temp;
        }

        private void DoEncrypt() {
            ApplyAddRoundKey(0);
            //StateDump();
            for (int i = 1; i < 10; ++i) {
                ApplyShiftRows();
                ApplySubByteMixColumn();
                //ApplySubByte();
                //ApplyMixColumn();
                ApplyAddRoundKey(4 * i);
            }
            ApplyShiftRows();
            ApplySubByte();
            ApplyAddRoundKey(4 * 10);
        }

        public UInt32[] Decrypt(UInt32[] CipherText) {
            if (CipherText.Length != 4) {
                throw new System.Exception("Invalid length of argument array in Encrypt");
            }
            else {
                for (int i = 0; i < 4; ++i) {
                    this.IntermediateData[i] = CipherText[i];
                }
                DoDecrypt();
            }
            UInt32[] temp = new UInt32[4];
            for (int i = 0; i < 4; ++i) temp[i] = this.IntermediateData[i];
            return temp;
        }

        private void DoDecrypt() {
            ApplyAddRoundKey(40);
            //StateDump();
            for (int i = 9; i > 0; --i) {
                ApplyInvShiftRows();
                //ApplyInvSubByte();
                //ApplyInvMixColumn();
                ApplyInvSubByteMixColumn();
                ApplyInvAddRoundKey(4 * i);
            }
            ApplyInvSubByte();
            ApplyInvShiftRows();
            ApplyAddRoundKey(0);
        }
        #endregion

				#region Functions related to key-scheduling
				public void SetKey(UInt32[] SecretKey) {
            if (SecretKey.Length != 4) {
                throw new System.Exception("Invalid length of argument array in SetKey");
            }
            else {
                for (int i = 0; i < 4; ++i) {
                    this.SecretKey[i] = SecretKey[i];
                }
                KeySchedule();
            }
        }

        public void PrintKey() {
            Console.WriteLine(Utility.UInt32ArrayToString(SecretKey));
        }

        private void KeySchedule() {
            const int Nk = 4;
            UInt32 temp;

            for (int i = 0; i < Nk; ++i) {
                RoundKey[i] = SecretKey[i];
            }
            for (int i = Nk; i < 44; ++i) {
                temp = RoundKey[i - 1];
                if (i % Nk == 0) {
                    //temp = SubWord(RotWord(temp)) ^ Rcon[(i / Nk) - 1];
                    temp = SubWord(Utility.RotateLeft(temp, 8)) ^ Rcon[(i / Nk) - 1];
                }
                RoundKey[i] = RoundKey[i - Nk] ^ temp;
            }

            // Generating decrypt round keys
            for (int i = 0; i < 4; ++i) {
                InvRoundKey[i] = RoundKey[i];
            }
            for (int i = 4; i < 40; ++i) {
                InvRoundKey[i] = InvMixColumn(RoundKey[i]);
            }
            for (int i = 40; i < 44; ++i) {
                InvRoundKey[i] = RoundKey[i];
            }
        }

        public void PrintRoundKey() {
            string tmp = "Round Keys:\n";
            for (int i = 0; i < 44; ++i) {
                tmp += string.Format("    {0}: {1:X8}\n", i, RoundKey[i]);
            }
					System.Diagnostics.Debug.WriteLine(tmp);
//            Console.WriteLine(tmp);
        }
        #endregion

				#region AES functions for encryption
				public Byte SubByte(Byte a) {
            return Sbox[a];
        }

        public UInt32 SubWord(UInt32 x) {
            Byte[] ByteArray;
            ByteArray = Utility.UInt32ToByteArray(x);
            for (int i = 0; i < 4; ++i) {
                ByteArray[i] = Sbox[ByteArray[i]];
            }
            return Utility.ByteArrayToUInt32(ByteArray);
        }

        public UInt32 MixColumn(UInt32 x) {
            Byte[] ba;
            Byte[] temp = new Byte[4]{0, 0, 0, 0};

            ba = Utility.UInt32ToByteArray(x);
            temp[0] = (Byte)(MultGF(ba[0], (Byte)0x02) ^ MultGF(ba[1], (Byte)0x03) ^ ba[2] ^ ba[3]);
            temp[1] = (Byte)(ba[0] ^ MultGF(ba[1], (Byte)0x02) ^ MultGF(ba[2], (Byte)0x03) ^ ba[3]);
            temp[2] = (Byte)(ba[0] ^ ba[1] ^ MultGF(ba[2], (Byte)0x02) ^ MultGF(ba[3], (Byte)0x03));
            temp[3] = (Byte)(MultGF(ba[0], (Byte)0x03) ^ ba[1] ^ ba[2] ^ MultGF(ba[3], (Byte)0x02));
            return Utility.ByteArrayToUInt32(temp);
        }

        public UInt32 SubByteMixColumn(UInt32 x) {
            UInt32 tmp;
            Byte[] ByteArray;
            ByteArray = Utility.UInt32ToByteArray(x);

            tmp = Utility.RotateRight(Tbox[ByteArray[3]], 8);
            tmp = Utility.RotateRight(tmp ^ Tbox[ByteArray[2]], 8);
            tmp = Utility.RotateRight(tmp ^ Tbox[ByteArray[1]], 8);

            tmp = tmp ^ Tbox[ByteArray[0]];
            return tmp;
        }

        public UInt32[] ShiftRows(UInt32[] UInt32Array) {
            Byte[,] State;
            Byte[] ByteArray = new Byte[4];;
            UInt32[] ResultArray = new UInt32[4];

            // Transfer Uint32[4] array to the Byte[4, 4] array
            State = UInt32ArrayToState(UInt32Array);

            // Reconstruct UInt32[4] array applying ShiftRow transformation
            for (int column = 0; column < 4; ++column) {
                for (int row = 0; row < 4; ++row) {
                    ByteArray[row] = State[row, (column + row) % 4];
                }
                ResultArray[column] = Utility.ByteArrayToUInt32(ByteArray);
            }
            return ResultArray;
        }
        #endregion

				#region AES functions for decryption
				public Byte InvSubByte(Byte a) {
            return InvSbox[a];
        }

        public UInt32 InvSubWord(UInt32 x) {
            Byte[] ByteArray;
            ByteArray = Utility.UInt32ToByteArray(x);
            for (int i = 0; i < 4; ++i) {
                ByteArray[i] = InvSbox[ByteArray[i]];
            }
            return Utility.ByteArrayToUInt32(ByteArray);
        }

        public UInt32 InvMixColumn(UInt32 x) {
            Byte[] ba;
            Byte[] temp = new Byte[4] { 0, 0, 0, 0 };

            ba = Utility.UInt32ToByteArray(x);
            temp[0] = (Byte)(MultGF(ba[0], (Byte)0x0e) ^ MultGF(ba[1], (Byte)0x0b)
                           ^ MultGF(ba[2], (Byte)0x0d) ^ MultGF(ba[3], (Byte)0x09));
            temp[1] = (Byte)(MultGF(ba[0], (Byte)0x09) ^ MultGF(ba[1], (Byte)0x0e)
                           ^ MultGF(ba[2], (Byte)0x0b) ^ MultGF(ba[3], (Byte)0x0d));
            temp[2] = (Byte)(MultGF(ba[0], (Byte)0x0d) ^ MultGF(ba[1], (Byte)0x09)
                           ^ MultGF(ba[2], (Byte)0x0e) ^ MultGF(ba[3], (Byte)0x0b));
            temp[3] = (Byte)(MultGF(ba[0], (Byte)0x0b) ^ MultGF(ba[1], (Byte)0x0d)
                           ^ MultGF(ba[2], (Byte)0x09) ^ MultGF(ba[3], (Byte)0x0e));

            return Utility.ByteArrayToUInt32(temp);
        }

        public UInt32 InvSubByteMixColumn(UInt32 x) {
            UInt32 tmp;
            Byte[] ByteArray;
            ByteArray = Utility.UInt32ToByteArray(x);

            tmp = Utility.RotateRight(InvTbox[ByteArray[3]], 8);
            tmp = Utility.RotateRight(tmp ^ InvTbox[ByteArray[2]], 8);
            tmp = Utility.RotateRight(tmp ^ InvTbox[ByteArray[1]], 8);

            tmp = tmp ^ InvTbox[ByteArray[0]];
            return tmp;
        }

        public UInt32[] InvShiftRows(UInt32[] UInt32Array) {
            Byte[,] State;
            Byte[] ByteArray = new Byte[4];
            UInt32[] ResultArray = new UInt32[4];

            // Transfer Uint32[4] array to the Byte[4, 4] array
            State = UInt32ArrayToState(UInt32Array);

            // Reconstruct UInt32[4] array applying InvShiftRow 
            for (int column = 0; column < 4; ++column) {
                for (int row = 0; row < 4; ++row) {
                    ByteArray[row] = State[row, (column - row + 4) % 4];
                }
                ResultArray[column] = Utility.ByteArrayToUInt32(ByteArray);
            }
            return ResultArray;
        }
        #endregion

				#region Utility functions

				// Multiplication over a GF(2^8) with
        // p(x) = x^8 + x^4 + x^3 + x + 1 = 0x11b
        public Byte MultGF(Byte x, Byte y) {
            Byte temp = 0;
            while (y != 0) {
                if ((y & 0x01) == 0x01) temp ^= x;
                x = ((x & 0x80) == 0x80) ?
                        (Byte)((x << 1) ^ 0x1b) : (Byte)(x << 1); // Doubling
                y = (Byte)(y >> 1);
            }
            return temp;
        }

        public Byte[,] UInt32ArrayToState(UInt32[] UInt32Array) {
            Byte[,] State = new Byte[4, 4];
            Byte[] ByteArray;
            for (int column = 0; column < 4; ++column) {
                ByteArray = Utility.UInt32ToByteArray(UInt32Array[column]);
                for (int row = 0; row < 4; ++row) {
                    State[row, column] = ByteArray[row];
                }
            }
            return State;
        }

        // Break the this.IntermediateData[] into a matrix of state.
        public Byte[,] GetState() {
            return UInt32ArrayToState(IntermediateData);
        }

        public void StateDump() {
            Byte[,] State;
            State = GetState();
            string tmp = "";
            for (int row = 0; row < 4; ++row) {
                for (int column = 0; column < 4; ++column) {
                    tmp += string.Format("{0:x2} ", State[row, column]);
                }
                tmp += "\n";
            }
            Console.WriteLine(tmp);
        }
        #endregion
	}
}
