﻿/*-------------------------------------------------------------------------
 SASEBO cipher module setting
 
 File name   : Controller.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : Aug/18/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008, 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.ComponentModel;

namespace SASEBO_Checker_AES {
  //================================================================ Controller
  class Controller
  {
    //************************************************ Variable
    SASEBO.CipherModule module;
    Tracer tracer;


		//************************************************ Method
    //------------------------------------------------ Constructor
    public Controller() {}

		//------------------------------------------------ module_open()
    public int module_setup(int idx_if, int idx_lsi)
    {
      if (module!=null) module.close();

      SASEBO.ControlInterface ctrlif = SASEBO.ControlInterfaceFactory.getInstance(idx_if);
      module = CipherModuleFactory.getInstance(idx_lsi, ctrlif);
      int lsi_version = module.open();
      return lsi_version;
    }

		//------------------------------------------------ module_setup()
    //# public void tracer_setup(int idx_lsi, string num_trace, string key_st)
    public void tracer_setup(int idx_lsi, string num_trace, string message_st, int hash_loop, int check_latency, int data_size)
    {
        //tracer = TracerFactory.getInstance(idx_lsi, module, Convert.ToInt32(num_trace), key_st);
        tracer = TracerFactory.getInstance(idx_lsi, module, Convert.ToInt32(num_trace), message_st, hash_loop, check_latency, data_size);
    }

		//------------------------------------------------ module_addCompletedEventHandler()
    public void tracer_addCompletedEventHandler (RunWorkerCompletedEventHandler handler) {
      tracer.addBGWorker_CompletedEventHandler(handler);
    }

		//------------------------------------------------ module_addProgressChangedEventHandler()
    public void tracer_addProgressChangedEventHandler (ProgressChangedEventHandler handler) {
      tracer.addBGWorker_ProgressChangedEventHandler(handler);
    }

        //------------------------------------------------ module_addProgressChangedEventHandler2()
    public void tracer_addProgressChangedEventHandler2(ProgressChangedEventHandler handler) {
        tracer.addBGWorker_ProgressChangedEventHandler2(handler);
    }

		//------------------------------------------------ module_run()
    public void tracer_run()    { tracer.run(); }

		//------------------------------------------------ module_cancel()
    public void tracer_cancel() { tracer.cancel(); }

  }


	//================================================================ CipherModuleFactory
  public class CipherModuleFactory
  {
    public static SASEBO.CipherModule getInstance(int id, SASEBO.ControlInterface ctrlif)
    {
      switch (id) {
        case 0: return new SASEBO.CipherModule_rev1(ctrlif);
        //case 1: return new SASEBO.CipherModule_rev2(ctrlif);
        default: return null;
      }
    }
  }



	//================================================================ TracerFactory
  public class TracerFactory
  {
      //# public static Tracer getInstance(int idx, SASEBO.CipherModule module, int num_trace, string key_st)
      public static Tracer getInstance(int idx, SASEBO.CipherModule module, int num_trace, string message_st, int hash_loop, int check_latency, int data_size)
    {
      switch (idx) {
          //# case 0: return new Tracer_LSI_rev1(module, Convert.ToInt32(num_trace), key_st);
          //# case 1: return new Tracer_LSI_rev2(module, Convert.ToInt32(num_trace), key_st);
          case 0: return new Tracer_LSI_rev1(module, Convert.ToInt32(num_trace), message_st, hash_loop, check_latency, data_size);
          //case 1: return new Tracer_LSI_rev2(module, Convert.ToInt32(num_trace), message_st);
          default: return null;
      }
    }
  }
}
