﻿/*-------------------------------------------------------------------------
SASEBO_Checker_AES

File name   : Form_Controller.cs
Version     : Version 1.0
Created     : Aug/18/2009
Last update : Aug/18/2009
Desgined by : Toshihiro Katashita


Copyright (C) 2009 AIST

By using this code, you agree to the following terms and conditions.

This code is copyrighted by AIST ("us").

Permission is hereby granted to copy, reproduce, redistribute or
otherwise use this code as long as: there is no monetary profit gained
specifically from the use or reproduction of this code, it is not sold,
rented, traded or otherwise marketed, and this copyright notice is
included prominently in any copy made.

We shall not be liable for any damages, including without limitation
direct, indirect, incidental, special or consequential damages arising
from the use of this code.

When you publish any results arising from the use of this code, we will
appreciate it if you can cite our webpage
(http://www.rcis.aist.go.jp/special/SASEBO/).
-------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SASEBO_Checker_AES
{
	//================================================================ Form_Controller
	public partial class Form_Controller : Form
	{
		//************************************************ Variable
		Controller ctrl;
		Random rand;
		int randomseed = 1;
		int hash_loop = 0; //#
		int check_latency; //#
		int data_size; //#
		string input_text;

		//************************************************ Method
		//------------------------------------------------ Constructor
		public Form_Controller()
		{
			this.StartPosition = FormStartPosition.Manual;
			InitializeComponent();

			rand = new Random(randomseed);
		}

		//------------------------------------------------ Initialization
		private void Form_Controller_Load(object sender, EventArgs e)
		{
			//form_trace.StartPosition = FormStartPosition.Manual;
			cb_if.SelectedIndex  = 0;
			//cb_lsi.SelectedIndex = 0;
			cb_algorithm.SelectedIndex = 0;
			ctrl = new Controller();

		}

		//************************************************ Control
		//------------------------------------------------ Button, Start
		private void button_start_Click(object sender, EventArgs e)
		{
			if (button_start.Text == "Start")
			{
				button_start.Text = "Stop";

				//int version = ctrl.module_setup(cb_if.SelectedIndex, cb_lsi.SelectedIndex);
				int version = ctrl.module_setup(cb_if.SelectedIndex, 0); //cb_lsi.SelectedIndex == 0 に固定．
				Console.WriteLine(version.ToString("X4"));

				if (cb_algorithm.SelectedIndex == 0) { data_size = 64; } //SHA-256
				else if (cb_algorithm.SelectedIndex == 1) { data_size = 32 + 4; } //Skein
				else if (cb_algorithm.SelectedIndex == 2) { data_size = 32; } //CubeHash
                else if (cb_algorithm.SelectedIndex == 3) { data_size = 64 + 2; } //BLAKE
                else if (cb_algorithm.SelectedIndex == 4) { data_size = 64 * 3 + 2; } //ECHO
                else if (cb_algorithm.SelectedIndex == 5) { data_size = 64; } //Groustl
                else if (cb_algorithm.SelectedIndex == 6) { data_size = 4; } //Hamsi
                else if (cb_algorithm.SelectedIndex == 7) { data_size = 64; } //Shabal
                else if (cb_algorithm.SelectedIndex == 8) { data_size = 32; } //Luffa


				hash_loop++; //# messageを送った回数．
				if (cb_latency.Checked) {
					tbox_numhash.Text = "1";
					check_latency = 1;
				} else {
					check_latency = 0;
				}
				//# ctrl.tracer_setup(cb_lsi.SelectedIndex, tbox_numtrace.Text, tb_key.Text);
				//# ctrl.tracer_setup(cb_lsi.SelectedIndex, tbox_numtrace.Text, tb_key.Text, trace_loop);
				//ctrl.tracer_setup(cb_lsi.SelectedIndex, tbox_numhash.Text, tb_message.Text, hash_loop, check_latency);
                input_text = tb_message.Text + " " + tb_message2.Text + " " + textBox1.Text + " " + textBox2.Text + " " + textBox3.Text + " " + textBox4.Text + " " + textBox5.Text;
				ctrl.tracer_setup(0, tbox_numhash.Text, input_text, hash_loop, check_latency, data_size); //cb_lsi.SelectedIndex == 0 に固定．
				ctrl.tracer_addCompletedEventHandler(new RunWorkerCompletedEventHandler(worker_RunWorkerCompleted));
				ctrl.tracer_addProgressChangedEventHandler(new ProgressChangedEventHandler(worker_ProgressChanged));
				ctrl.tracer_run();
				//# if (trace_loop == Convert.ToInt32(tbox_numtrace.Text)) trace_loop = 0;
				if (hash_loop == Convert.ToInt32(tbox_numhash.Text)) hash_loop = 0;
			}
			else
			{
				ctrl.tracer_cancel();
			}
		}

		//------------------------------------------------ Menu, Quit
		private void quitToolStripMenuItem_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		//************************************************ BackGroundWorker_EventHandler
		//------------------------------------------------ 
		private void worker_ProgressChanged(object sender, ProgressChangedEventArgs e) {
			//# label_numtrace.Text = ((TracerReport)e.UserState).num_trace.ToString();
			label_numhash.Text = ((TracerReport)e.UserState).num_hash.ToString();

			//# modified by Kobayashi.
			label_text_in.Text = format(((TracerReport)e.UserState).text_in);
			label_text_out.Text = format(((TracerReport)e.UserState).text_out);
			//label_text_ans.Text = format(((TracerReport)e.UserState).text_ans);

			if (cb_latency.Checked)
			{
				label_text_latency_core.Text = format2(((TracerReport)e.UserState).text_latency_core); //output latency
				label_text_latency_all.Text = format3(((TracerReport)e.UserState).text_latency_all); //output latency
			}
		}

		//------------------------------------------------ 
		private void worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			/*
			   if (!e.Cancelled)
			   { // finished
			   MessageBox.Show("Finished.", "Notice", MessageBoxButtons.OK, MessageBoxIcon.Information);
			   }
			   else
			   {
			   MessageBox.Show("Cancelled.", "Notice", MessageBoxButtons.OK, MessageBoxIcon.Stop);
			   }
			   */
			button_start.Text = "Start";
		}


		//************************************************ 
		//------------------------------------------------ format()
		private string format(byte[] data)
		{
			string text = "";
			for (int i = 0; i < data.Length; i++)
			{
				text += String.Format("{0:X2} ", data[i]);
				if (i % 32 == 31)
				{
					text += "\r\n";
				}
			}
			return text;
		}

		//------------------------------------------------ format2()
		private string format2(byte[] data)
		{
			string text = "";
			//short j = BitConverter.ToInt16(data, 0);

			//text = String.Format("{0:D5}", (data[0] | (data[1] << 8)));
			text = String.Format("{0:D5}", (data[0] * 256 + (data[1] - 3)));
			//fetch を立ててから cnt が止まるまで 3 clk 必要なため，Latency から 3 を引く．
			text += " cycle";

			return text;
		}

		//------------------------------------------------ format3()
		private string format3(byte[] data)
		{
			string text = "";

			text = String.Format("{0:D5}", (data[0] * 256 + (data[1] + 1)));
			text += " cycle";

			return text;
		}

        private void lb_fixed_ciphertext_Click(object sender, EventArgs e)
        {

        }

        private void label_text_latency_all_Click(object sender, EventArgs e)
        {

        }

        private void lb_fixed_latency_all_Click(object sender, EventArgs e)
        {

        }

        private void lb_fixed_latency_core_Click(object sender, EventArgs e)
        {

        }

        private void label_text_latency_core_Click(object sender, EventArgs e)
        {

        }

        private void label_text_out_Click(object sender, EventArgs e)
        {

        }

        private void label_text_in_Click(object sender, EventArgs e)
        {

        }

	}
}
