﻿/*-------------------------------------------------------------------------
 SASEBO Cipher Module; LSI GAIA(130nm)
 
 File name   : SASEBO_CipherModule.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : Jun/11/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008, 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace SASEBO
{
	//================================================================ CipherModule_rev1
	public class CipherModule_rev1 : CipherModule
	{
		//************************************************ Variable
		// lbus address
        //public int ADDR_CONT = 0x0002;
        public int ADDR_CONT = 0x1000;
        public int ADDR_IPSEL = 0x0004;
    public int ADDR_OUTSEL   = 0x0008;
    //public int ADDR_MODE = 0x000C;
    public int ADDR_ENCDEC = 0x2000;
    public int ADDR_RSEL = 0x000E;
		public int ADDR_KEY0     = 0x0100;
        //public int ADDR_ITEXT0 = 0x0140;
        public int ADDR_ITEXT0 = 0x0000;
        //public int ADDR_OTEXT0 = 0x0180;
        //public int ADDR_OTEXT0 = 0x0020;
        //public int ADDR_OTEXT0 = 0x0100;
        public int ADDR_OTEXT0 = 0x0080;
        public int ADDR_LATENCY_CORE = 0x3000; // address of latency
        public int ADDR_LATENCY_ALL = 0x3200; // address of latency
        public int ADDR_EXP00 = 0x0200;
    public int ADDR_MOD00    = 0x0300;
    public int ADDR_IDATA00  = 0x0400;
		public int ADDR_ODATA00  = 0x0500;

        //public int data_size = 0x0200; //# 32 bit
        //public int data_size = 0x0800; //# 128 bit
        //public int data_size = 0x1000; //# 256 bit
        //public int data_size = 0x1800; //# 384 bit
        //public int data_size = 0x2000; //# 512 bit
        
		public CipherCore[] cores = new CipherCore[] {
      // IPSEL0
			new CipherCore("AES_Comp",     "AES",      0x00001), // AES0
			new CipherCore("AES_Comp_ENC", "AES",      0x00002), // AES1
			new CipherCore("AES_TBL",      "AES",      0x00004), // AES2
			new CipherCore("AES_PPRM1",    "AES",      0x00008), // AES3
			new CipherCore("AES_PPRM3",    "AES",      0x00010), // AES4
			new CipherCore("DES",          "DES",      0x00020),
			new CipherCore("MISTY1",       "MISTY1",   0x00040),
			new CipherCore("Camellia",     "Camellia", 0x00080),
			new CipherCore("SEED",         "SEED",     0x00100),
			new CipherCore("CAST128",      "CAST128",  0x00200)
		};

		// cipher mode
        public int MODE_ENC = 0x0000;
		public int MODE_DEC = 0x0001;

		//LocalBus lbus;

        public int latency_check;

		//************************************************ Method
		//------------------------------------------------ Constructor
		public CipherModule_rev1(ControlInterface ctrlif)
		{
			lbus = new LocalBus(ctrlif);
		}
    
		//------------------------------------------------ select()
		public void select(int cipher)
		{
      // Select cipher
      if ((cipher & 0x10000) == 0) {
        lbus.write(ADDR_IPSEL,        cipher&0xFFFF);
        lbus.write(ADDR_IPSEL+0x0002, 0);
      } else {
        lbus.write(ADDR_IPSEL,        0);
        lbus.write(ADDR_IPSEL+0x0002, cipher&0xFFFF);
      }

      // Reset
			lbus.write(ADDR_CONT,   0x0004);
			lbus.write(ADDR_CONT,   0x0000);

      // Select output
      if ((cipher & 0x10000) == 0) {
        lbus.write(ADDR_OUTSEL,        cipher&0xFFFF);
        lbus.write(ADDR_OUTSEL+0x0002, 0);
      } else {
        lbus.write(ADDR_OUTSEL,        0);
        lbus.write(ADDR_OUTSEL+0x0002, cipher&0xFFFF);
      }
    }

		//------------------------------------------------ encdec()
		public void encdec(int encdec)
		{
            //lbus.write(ADDR_MODE, encdec);
            //lbus.write(ADDR_ENCDEC, encdec);
            //lbus.write(ADDR_CONT, (data_size + 0x0010));
        }

		//------------------------------------------------ key(): Set cipher key to the cipher module.
		public void key(byte[] key, int len)
		{
            //lbus.writeBurst(ADDR_KEY0, key, len);
            //lbus.write(ADDR_CONT, 0x0002);  // Execute key generation
            //while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
		}

		//------------------------------------------------ write(): Write plaintext/ciphertext to the cipher module.
		public void write(byte[] dat, int len)
		{
			lbus.writeBurst(ADDR_ITEXT0, dat, len);
            lbus.write(ADDR_CONT, 0x0001); // Execute cipher processing
            //lbus.write(0x3000, 0x0001); // Execute lbus_drdy
		}

        //------------ next_write (): Write next 512-bit message
        public void next_write(byte[] dat, int len, int EOM, int check_latency)
        {
            //while (lbus.read(ADDR_CONT) != 0x0000) { /*nop*/ } //# polling.

            //lbus.write(ADDR_CONT, (data_size + EOM)); //修正前コード．
            lbus.write(ADDR_CONT, ((0x0100 * len /2) + EOM));
            //lbus.write(ADDR_CONT, data_size + EOM + 0x0020); //0x0020 : check_latencyフラグを立たせる信号．不要
            
            //#while ((lbus.read(ADDR_CONT) != 0x0003) && (lbus.read(ADDR_CONT) != 0x0000)) { /*nop*/ } //# polling. 

            //while ((lbus.read(ADDR_CONT) != (data_size + EOM + 0x0002)) && (lbus.read(ADDR_CONT) != data_size + EOM)) { /*nop*/ } //# polling. 修正前コード．
            //while (lbus.read(ADDR_CONT) != data_size + EOM) { /*nop*/ } //# polling. num_hash=1の時しか考えていない．
            //# 0x0000 meets in first message. 0x0003 meets for second to #Trace round.
            
            lbus.writeBurst(ADDR_ITEXT0, dat, len);
            //#lbus.write(ADDR_CONT, 0x0001); // Execute cipher processing

            //lbus.write(ADDR_CONT, (data_size + EOM + 0x0001)); // Execute cipher processing //修正前コード
            if (check_latency == 1)
            {
                //lbus.write(ADDR_CONT, (data_size + EOM + 0x0001 + 0x0020)); // Execute cipher processing //latency信号を立たせる．
                lbus.write(ADDR_CONT, ((0x0100 * len / 2) + EOM + 0x0001 + 0x0020)); // Execute cipher processing //latency信号を立たせる．
            }
            else
            {
                //lbus.write(ADDR_CONT, (data_size + EOM + 0x0001)); // Execute cipher processing
                lbus.write(ADDR_CONT, ((0x0100 * len / 2) + EOM + 0x0001)); // Execute cipher processing
            }
        }

        //------------------------------------------------ read():  Read ciphertext/plaintext from the cipher module.
        public void read(byte[] dat, int len, int EOM, int len2) //修正前コード
        //public void read(byte[] dat, byte[] dat2, int len, int EOM, int latency_check)
        {
            //lbus.write(ADDR_CONT, (data_size + 0x0008 + EOM)); //# ctrl = 1000. //修正前コード
            lbus.write(ADDR_CONT, ((0x0100 * len2 / 2) + 0x0008 + EOM)); //# ctrl = 1000. //修正前コード
            //lbus.write(ADDR_CONT, (lbus.read(ADDR_CONT) + 0x0008)); //# ctrl = 1000. 

            //while (lbus.read(ADDR_CONT) != (data_size + 0x0000 + EOM)) { /*nop*/ } //# polling.
            while (lbus.read(ADDR_CONT) != ((0x0100 * len2 / 2) + 0x0000 + EOM)) { /*nop*/ } //# polling.
            //while (lbus.read(ADDR_CONT) != data_size + 0x0010 + 0x0000) { /*nop*/ } //# polling.
            //# wait while, ctrl[3] = 1.
            lbus.readBurst(ADDR_OTEXT0, dat, len);
            lbus.write(ADDR_CONT, 0x0000);
        }

        //------------------------------------------------ read2():  Read ciphertext/plaintext from the cipher module.
        //public void read(byte[] dat, int len, int EOM) //修正前コード
        public void read2(byte[] dat, byte[] dat2, byte[] dat3, int len, int EOM, int len2)
        {
            
                //lbus.write(ADDR_CONT, (data_size + 0x0008 + EOM)); //# ctrl = 1000. //修正前コード
            
            //lbus.write(ADDR_CONT, (lbus.read(ADDR_CONT) + 0x0008)); //# ctrl = 1000. 

            //while (lbus.read(ADDR_CONT) != (data_size + 0x0000 + EOM)) { /*nop*/ } //# polling.
            while (lbus.read(ADDR_CONT) != ((0x0100 * len2 / 2) + 0x0000 + EOM)) { /*nop*/ } //# polling.
            //while (lbus.read(ADDR_CONT) != data_size + 0x0010 + 0x0000) { /*nop*/ } //# polling.
            //# wait while, ctrl[3] = 1.
            lbus.readBurst(ADDR_OTEXT0, dat, len);
            lbus.readBurst(ADDR_LATENCY_CORE, dat2, 2); //追加コード
            lbus.readBurst(ADDR_LATENCY_ALL, dat3, 2);
            
            lbus.write(ADDR_CONT, 0x0000);
        }
        
        //------------------------------------------------ rsa_key(): Set cipher key to the cipher module.
		public void rsa_key(byte[] key, byte[] mod, int len)
		{
			lbus.writeBurst(ADDR_MOD00, mod, len);
			lbus.writeBurst(ADDR_EXP00, key, len);
      lbus.write(ADDR_CONT, 0x0002);  // Read key and modular.
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
		}

		//------------------------------------------------ rsa_write(): Write text to the cipher module.
		public void rsa_write(byte[] dat, int len)
		{
			lbus.writeBurst(ADDR_IDATA00, dat, len);
			lbus.write(ADDR_CONT, 0x0001); // Execute cipher processing
		}

		//------------------------------------------------ rsa_read():  Read text from the cipher module.
		public void rsa_read(byte[] dat, int len)
		{
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
			lbus.readBurst(ADDR_ODATA00, dat, len);
		}

		//------------------------------------------------ convertDESKey()
		public byte[] convertDESKey(byte[] k)
		{
			UInt64 key = 0;
			byte[] kr = new byte[16];

			for (int i = 0; i < 8; i++) key += (UInt64)(((UInt64)(k[i + 8] & 0xFE) >> 1) << ((7 - i) * 7));
      //for (int i = 0; i < 8; i++) key += (UInt64)(((UInt64)(k[i + 8] & 0xEF)) << ((7 - i) * 7));
      //System.Diagnostics.Debug.WriteLine(key.ToString("X16"));
			for (int i = 0; i < 16; i++) kr[i] = 0;
			for (int i = 0; i < 8; i++) kr[i + 8] = (byte)((key >> ((7 - i) * 8)) & 0xFF);
			return kr;
		}

        //------------------------------------------------ read_ctrl()
        /*public void read_ctrl(int addr, byte[] dat
            )
        {
            lbus.read(addr);
        }*/

	}
}
