﻿/*-------------------------------------------------------------------------
 SASEBO Cipher Module; LSI MASH(130nm) and CHAR(90nm)
 
 File name   : SASEBO_CipherModule.cs
 Version     : Version 1.0
 Created     : Aug/20/2008
 Last update : May/18/2009
 Desgined by : Toshihiro Katashita
 
 
 Copyright (C) 2008, 2009 AIST
 
 By using this code, you agree to the following terms and conditions.
 
 This code is copyrighted by AIST ("us").
 
 Permission is hereby granted to copy, reproduce, redistribute or
 otherwise use this code as long as: there is no monetary profit gained
 specifically from the use or reproduction of this code, it is not sold,
 rented, traded or otherwise marketed, and this copyright notice is
 included prominently in any copy made.
 
 We shall not be liable for any damages, including without limitation
 direct, indirect, incidental, special or consequential damages arising
 from the use of this code.
 
 When you publish any results arising from the use of this code, we will
 appreciate it if you can cite our webpage
 (http://www.rcis.aist.go.jp/special/SASEBO/).
 -------------------------------------------------------------------------*/ 

using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

namespace SASEBO
{
	//================================================================ CipherModule
	public class CipherModule_rev2 : CipherModule
	{
		//************************************************ Variable
		// lbus address
		public int ADDR_CONT     = 0x0002;
    public int ADDR_IPSEL    = 0x0004;
    public int ADDR_OUTSEL   = 0x0008;
    public int ADDR_MODE     = 0x000C;
		public int ADDR_RSEL     = 0x000E;
		public int ADDR_KEY0     = 0x0100;
		public int ADDR_IV0      = 0x0110;
    public int ADDR_ITEXT0   = 0x0120;
    public int ADDR_RAND0    = 0x0160;
		public int ADDR_OTEXT0   = 0x0180;
    public int ADDR_RDATA0   = 0x01C0;
    public int ADDR_RKEY0    = 0x01D0;
    public int ADDR_EXP00    = 0x0200;
    public int ADDR_MOD00    = 0x0300;
    public int ADDR_PREDAT00 = 0x0340;
    public int ADDR_IDATA00  = 0x0400;
		public int ADDR_ODATA00  = 0x0500;

		public CipherCore[] cores = new CipherCore[] {
      // IPSEL0
			new CipherCore("AES_Comp",     "AES",      0x00001), // AES0
			new CipherCore("AES_TBL",      "AES",      0x00002), // AES1
			new CipherCore("AES_PPRM1",    "AES",      0x00004), // AES2
			new CipherCore("AES_PPRM3",    "AES",      0x00008), // AES3
			new CipherCore("AES_Comp_ENC", "AES",      0x00010), // AES4
			new CipherCore("AES_CTR_Pipe", "AES",      0x00020), // AES5
			new CipherCore("AES_FA",       "AES",      0x00040), // AES6
			new CipherCore("AES_PKG",      "AES",      0x00080), // AES7
			new CipherCore("AES_MAO",      "AES",      0x00100), // AES8
			new CipherCore("AES_MDPL",     "AES",      0x00200), // AES9
			new CipherCore("AES_TI",       "AES",      0x00400), // AES10
			new CipherCore("AES_WDDL",     "AES",      0x00800), // AES11
			new CipherCore("AES_RSL",      "AES",      0x01000), // AES12
			new CipherCore("AES_RSL2",     "AES",      0x02000), // AES13

      // IPSEL1
			new CipherCore("Camellia",     "Camellia", 0x10001),
			new CipherCore("SEED",         "SEED",     0x10002),
			new CipherCore("MISTY1",       "MISTY1",   0x10004),
			new CipherCore("TDES",         "DES",      0x10008),
			new CipherCore("DES",          "DES",      0x10010),
			new CipherCore("CAST128",      "CAST128",  0x10020),
			new CipherCore("ECC",          "ECC",      0x10040), 
			new CipherCore("RSA",          "RSA",      0x10080),
		};

		// cipher mode
		public int MODE_ENC     = 0x0000;
		public int MODE_DEC     = 0x0001;
		public int MODE_RSA_LB  = 0x0000;
		public int MODE_RSA_RB  = 0x0008;
		public int MODE_RSA_LBC = 0x0010;
		public int MODE_RSA_RBC = 0x0018;
		public int MODE_RSA_MPL = 0x0020;
		public int MODE_RSA_MJR = 0x0028;
		public int MODE_RSA_CRT = 0x0040;

		//volatile LocalBus lbus;


		//************************************************ Method
		//------------------------------------------------ Constructor
		public CipherModule_rev2(ControlInterface ctrlif)
		{
			lbus = new LocalBus(ctrlif);
		}
    
		//------------------------------------------------ select()
		public void select(int cipher)
		{
      // Select cipher
      if ((cipher & 0x10000) == 0) {
        lbus.write(ADDR_IPSEL,        cipher&0xFFFF);
        lbus.write(ADDR_IPSEL+0x0002, 0);
      } else {
        lbus.write(ADDR_IPSEL,        0);
        lbus.write(ADDR_IPSEL+0x0002, cipher&0xFFFF);
      }

      // Reset
			lbus.write(ADDR_CONT,   0x0004);
			lbus.write(ADDR_CONT,   0x0000);

      // Select output
      if ((cipher & 0x10000) == 0) {
        lbus.write(ADDR_OUTSEL,        cipher&0xFFFF);
        lbus.write(ADDR_OUTSEL+0x0002, 0);
      } else {
        lbus.write(ADDR_OUTSEL,        0);
        lbus.write(ADDR_OUTSEL+0x0002, cipher&0xFFFF);
      }

      // Reset AES-RSL
      lbus.write(0x0012, 0x0001);
      lbus.write(0x0012, 0x0000);
    }

		//------------------------------------------------ encdec()
		public void encdec(int encdec)
		{
			lbus.write(ADDR_MODE, encdec);
		}

		//------------------------------------------------ key(): Set cipher key to the cipher module.
		public void key(byte[] key, int len)
		{
			lbus.writeBurst(ADDR_KEY0, key, len);
			lbus.write(ADDR_CONT, 0x0002);  // Execute key generation
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
		}

		//------------------------------------------------ iv(): Set initial vector to the cipher module.
		public void iv(byte[] iv, int len)
		{
			lbus.writeBurst(ADDR_IV0, iv, len);
		}

		//------------------------------------------------ iv_run(): Set initial vector to the cipher module.
		public void iv_run(byte[] iv, int len)
		{
			lbus.writeBurst(ADDR_IV0, iv, len);
			lbus.write(ADDR_CONT, 0x0001);  // Execute key generation
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
		}

		//------------------------------------------------ rand(): Set random seed to the cipher module.
		public void rand(byte[] rand, int len)
		{
			lbus.writeBurst(ADDR_RAND0, rand, len);
		}

		//------------------------------------------------ write(): Write plaintext/ciphertext to the cipher module.
		public void write(byte[] dat, int len)
		{
			lbus.writeBurst(ADDR_ITEXT0, dat, len);
			lbus.write(ADDR_CONT, 0x0001); // Execute cipher processing
		}

		//------------------------------------------------ read():  Read ciphertext/plaintext from the cipher module.
		public void read(byte[] dat, int len)
		{
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
			lbus.readBurst(ADDR_OTEXT0, dat, len);
		}


		//------------------------------------------------ rsa_key(): Set cipher key to the cipher module.
		public void rsa_key(byte[] key, byte[] mod, int len)
		{
			lbus.writeBurst(ADDR_MOD00, mod, len);
			lbus.writeBurst(ADDR_EXP00, key, len);
      lbus.write(ADDR_CONT, 0x0002);  // Read key and modular.
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
		}

		//------------------------------------------------ rsa_key_crt(): Set cipher key to the cipher module.
		public void rsa_key_crt(byte[] key, byte[] mod, byte[] pre, int len)
		{
			lbus.writeBurst(ADDR_EXP00, key, len);
      lbus.writeBurst(ADDR_MOD00, mod, len);
      lbus.writeBurst(ADDR_PREDAT00, pre, len);
      lbus.write(ADDR_CONT, 0x0002);  // Read key and modular.
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
		}

		//------------------------------------------------ rsa_write(): Write text to the cipher module.
		public void rsa_write(byte[] dat, int len)
		{
			lbus.writeBurst(ADDR_IDATA00, dat, len);
			lbus.write(ADDR_CONT, 0x0001); // Execute cipher processing
		}

		//------------------------------------------------ rsa_read():  Read text from the cipher module.
		public void rsa_read(byte[] dat, int len)
		{
      while (lbus.read(ADDR_CONT) != 0) { /*nop*/ }
			lbus.readBurst(ADDR_ODATA00, dat, len);
		}

		//------------------------------------------------ convertDESKey()
		public byte[] convertDESKey(byte[] k)
		{
			UInt64 key = 0;
			byte[] kr = new byte[16];

			for (int i = 0; i < 8; i++) key += (UInt64)(((UInt64)(k[i + 8] & 0xFE) >> 1) << ((7 - i) * 7));
      //for (int i = 0; i < 8; i++) key += (UInt64)(((UInt64)(k[i + 8] & 0xEF)) << ((7 - i) * 7));
      //System.Diagnostics.Debug.WriteLine(key.ToString("X16"));
			for (int i = 0; i < 16; i++) kr[i] = 0;
			for (int i = 0; i < 8; i++) kr[i + 8] = (byte)((key >> ((7 - i) * 8)) & 0xFF);
			return kr;
		}
	}
}
